unit AddContextMenu;

{
Purpose:

To add a right-click menu item for a known file type.  Example, you have
Notepad associated with .TXT files, but you have writtem your own special
editor called TPS.  You want the user still to have Notepad open a .TXT
file when it is double-clicked (this avoids confusing the user!), but you
also want to have the user be able to use your TPS editor on all .TXT 
files under special circumstances.  They would right-click a .TXT file, 
and one of the menu items should be "Edit with TPS".

Non-purpose:

To create context menu items for files that do not already have a default
double-click action.

Calling this rotuine:

    add_context_menu ('Edit with TPS', '.txt');


Untested ideas:

    Troels Jakobsen wrote:

    Just call the method with 'Folder' in stead of '.txt'.

    Use '*' to associate to all files.



Revision history:

2002 Jan 12  First version
2002 Feb 07  Add try/except/end around the loop (for Windows XP)

Copyright  2002 by David J Taylor, Edinburgh
Web site:  www.satsignal.net
E-mail:    davidtaylor@writeme.com
}

interface

procedure add_context_menu (const command_name, extensions: string);


implementation

uses
  Windows, Classes, Registry, Forms;

procedure add_context_menu (const command_name, extensions: string);
var
  extns: TStringList;
  reg: TRegistry;
  idx: integer;
  name: string;
  command: string;
begin
  reg := TRegistry.Create;
  reg.RootKey := HKEY_CLASSES_ROOT;
  extns := TStringList.Create;
  extns.CommaText := extensions;
  // Build the command string we want to store
  command := '"' + Application.ExeName + '" "%1"';
  // Loop over extensions we can handle
  for idx := 0 to extns.Count - 1 do
    try
    // See if this extension is already known in HKCR
    if reg.OpenKeyReadOnly ('\' + extns.Strings [idx]) then
      begin
      name := reg.ReadString ('');   // Get the name of this type
      if (name <> '') then
        begin
        // If not blank, open this type's shell key, but don't create it
        if reg.OpenKey ('\' + name + '\shell', False) then
          // Try to create a new key called command_name
          // Note that for Delphi5 we need to set the access explicitly
          reg.Access := KEY_READ or KEY_WRITE;
          if reg.OpenKey (command_name, True) then
            begin
            // The default value will be displayed in the context menu
            reg.WriteString ('', '&' + command_name);
            // So now open the command key, creating it if required
            reg.Access := KEY_READ or KEY_WRITE;
            if reg.OpenKey ('command', True)
              // and write the command string as the default value
              then reg.WriteString ('', command);
            end;
        end;
      end;
    except
    end;
  extns.Free;
  reg.Free;
end;

end.
